@@ -13,8 +13,8 @@ void close_on_exec(int fd, int on)
 {
   int flags;
 
-  if ((flags = fcntl(fd, F_GETFD, 0)) < 0)
+  if ((flags = fcntl(fd, F_GETFD, 0)) == -1)
     abort();
-  if (fcntl(fd, F_SETFD, on ? flags | PATTERN : flags & ~PATTERN) < 0)
+  if (fcntl(fd, F_SETFD, on ? flags | PATTERN : flags & ~PATTERN) == -1)
     abort();
 }
