@@ -24,6 +24,9 @@ int debug = 0;
 
 int devnull = -1;
 
+int limit_openfd = 0;
+int count_openfd = 0;
+
 static void write_seedfile(void);
 
 static void open_seedfile(const char *seedfile)
@@ -271,6 +274,14 @@ int main(int argc, char *argv[])
   if (debug)
     fprintf(stdout, "Debugging enabled\n");
 
+  /*
+   * Obtain the maximum number of open filedescriptors and adjust the
+   * counter of actual open files. As always we have open stdin, stdout,
+   * and stderr.
+   */
+  limit_openfd = obtain_limit();
+  count_openfd = 3;
+
   /*
    * Open /dev/null
    */
@@ -279,6 +290,7 @@ int main(int argc, char *argv[])
     fprintf(stderr, "Couldn't open /dev/null: %s\n", strerror(errno));
     exit(EXIT_FAILURE);
   }
+  count_openfd++;
 
   /*
    * Now, initialize the PRNG (allocate the memory).
@@ -292,8 +304,10 @@ int main(int argc, char *argv[])
    * Start by reading back the seed saved from former runs, if usage of a
    * seedfile is wanted. 
    */
-  if (!killmode && seedpath)
+  if (!killmode && seedpath) {
     open_seedfile(seedpath);
+    count_openfd++;
+  }
 
   if (!(service_socket = malloc(numsock * sizeof(int)))) {
     fprintf(stderr, "Could not allocate memory: %s\n", strerror(errno));
@@ -309,6 +323,7 @@ int main(int argc, char *argv[])
     fprintf(stderr, "Could not create socket: %s\n", strerror(errno));
     exit(EXIT_FAILURE);
   }
+  count_openfd++;
   /*
    * Set up the sockets to service entropy requests
    */
@@ -366,6 +381,7 @@ int main(int argc, char *argv[])
 	else if (killmode) {
 	  killpeer(kill_socket);
 	  shutdown(kill_socket, SHUT_RDWR);
+	  count_openfd--;
 	  continue;
 	}
 	else {
@@ -375,6 +391,7 @@ int main(int argc, char *argv[])
 	  exit(EXIT_FAILURE);
 	}
 	shutdown(kill_socket, SHUT_RDWR);
+	count_openfd--;
 
 	/*
          * Now there is a socket without service listening on it: we can
@@ -431,6 +448,7 @@ int main(int argc, char *argv[])
      */
     non_blocking(service_socket[i], 1);
     close_on_exec(service_socket[i], 1);
+    count_openfd++;
   }
 
   if (killmode) {
@@ -462,8 +480,15 @@ int main(int argc, char *argv[])
     openlog("prngd", LOG_PID, LOG_DAEMON);
     syslog(LOG_INFO, "prngd %s (%s) started up for user %s", PRNGD_VERSION,
 	   PRNGD_DATE, username);
+    count_openfd++;
+    syslog(LOG_INFO, "Have %d out of %d filedescriptors open\n", count_openfd,
+	    limit_openfd);
+
   }
 
+  if (debug)
+    fprintf(stderr, "Have %d out of %d filedescriptors open\n", count_openfd,
+	    limit_openfd);
   /*
    * Maybe we have changed our pid, let's add some more of it. 
    */
