@@ -144,11 +144,17 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
     FD_ZERO(&wfds);
     FD_ZERO(&efds);
     max_fd = 0;
-    for (i = 0; i < numsock; i++)
-    {
-      FD_SET(service_socket[i], &rfds);
-      if (service_socket[i] > max_fd)
+    /*
+     * Do not arm select for the listener socket if we are not ready
+     * to accept another connection.
+     */
+    if (count_openfd < limit_openfd - OPEN_FD_RESERVE) {
+      for (i = 0; i < numsock; i++)
+      {
+        FD_SET(service_socket[i], &rfds);
+        if (service_socket[i] > max_fd)
 	max_fd = service_socket[i];
+      }
     }
     if (gather_fd != -1)
     {
@@ -204,12 +210,16 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
       last_seed_stat = time(0);
     }
 
+    if (num < 0)
+      continue;		/* Ignore rfds and friends, since invalid */
+
     if (gather_fd != -1) {
       if (FD_ISSET(gather_fd, &efds)) {
 	if (debug)
 	  fprintf(stderr, "Closing connection to gatherer (efds)\n");
 	close(gather_fd);
 	gather_fd = -1;
+	count_openfd--;
       }
       else if (FD_ISSET(gather_fd, &rfds)) {
 	/*
@@ -228,6 +238,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	  rand_add(&gatherer_status, sizeof(gatherer_status), 0.0);
 	  rand_add(&gatherer_seed, sizeof(seed_t), 0.25);
 	  gather_fd = -1;
+	  count_openfd--;
 	}
       }
       if (gather_fd == -1) {
@@ -253,6 +264,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
       if (gather_fd != -1) {
 	close(gather_fd);
 	gather_fd = -1;
+	count_openfd--;
 	gatherer_entry++;
 	if (entropy_source[gatherer_entry].path == NULL) {
 	  first_gather = 0;	/* done */
@@ -323,15 +335,19 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	 */
 	close(pipedes[1]);
 	gather_fd = pipedes[0];
+	count_openfd++;
 	non_blocking(gather_fd, 1);
 	gather_start = time(NULL);
-	fprintf(stderr, "Spawned gatherer: %s (%s",
+	if (debug)
+	{
+	  fprintf(stderr, "Spawned gatherer: %s (%s",
 		entropy_source[gatherer_entry].path,
 		entropy_source[gatherer_entry].args[0]);
-	i = 1;
-	while ((i < 5) && (entropy_source[gatherer_entry].args[i]))
-	  fprintf(stderr, " %s", entropy_source[gatherer_entry].args[i++]);
-	fprintf(stderr, ")\n");
+	  i = 1;
+	  while ((i < 5) && (entropy_source[gatherer_entry].args[i]))
+	    fprintf(stderr, " %s", entropy_source[gatherer_entry].args[i++]);
+	  fprintf(stderr, ")\n");
+	}
       }
       continue;
 
@@ -496,6 +512,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 	else if (egd_query[i].state == to_be_error_closed)
 	  syslog(LOG_INFO, "closing service fd %d for error.", egd_query[i].fd);
 	close(egd_query[i].fd);
+	count_openfd--;
 	memset(egd_query[i].buffer, 0, 256);	/* Remove delivered entropy */
 	if (i < max_query - 1)
 	{
@@ -509,6 +526,17 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 
     for (i = 0; i < numsock; i++)
     {
+      if (count_openfd >= limit_openfd - OPEN_FD_RESERVE) {
+	if (debug)
+	  fprintf(stderr,
+		  "Could not accept connection: no more slots (%d of %d)\n",
+		  count_openfd, limit_openfd - OPEN_FD_RESERVE);
+	else
+	  syslog(LOG_INFO,
+		  "Could not accept connection: no more slots (%d of %d)",
+		  count_openfd, limit_openfd - OPEN_FD_RESERVE);
+	continue;	/* do not accept new connection */
+      }
       if (FD_ISSET(service_socket[i], &rfds))
       {
         sockunlen = sizeof(sockun);
@@ -519,6 +547,7 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
 				 * Ignore and stay alive 
 				 */
 
+	count_openfd++;
 	non_blocking(accept_fd, 1);
 	close_on_exec(accept_fd, 1);
 
@@ -551,7 +580,8 @@ void main_loop(int *service_socket, int numsock, struct sockaddr_un sockun,
         max_query++;
         egd_query[max_query - 1].fd = accept_fd;
         egd_query[max_query - 1].state = wait_command;
-        fprintf(stderr, "Opening service fd %d\n", egd_query[max_query-1].fd);
+	if (debug)
+          fprintf(stderr, "Opening service fd %d\n", egd_query[max_query-1].fd);
       }
     }
 
